import sys
import os
# import logging
from waveshare_epd import epd4in2
import time
import calendar
import pytz
from datetime import date
from PIL import Image,ImageDraw,ImageFont
# import traceback
import requests, json, time, datetime
from requests.exceptions import ConnectionError

def setup_screen():
    # WEATHER DISPLAY FUNCTION
    try:
        # set up screen
        epd = epd4in2.EPD()
        epd.init()
        epd.Clear()
        epd.sleep()
    except IOError as e:
        print("EPD error:", e)
    except KeyboardInterrupt:
        epd4in2.epdconfig.module_exit()
        exit()
    return epd

def convert_time(time):
    time = time - (3660 * 6)
    return datetime.datetime.fromtimestamp(int(time)).strftime('%-I:%M')

def convert_hour(time):
    time = time - (3660 * 6)
    return datetime.datetime.fromtimestamp(int(time)).strftime('%-I%p')

def useCurrentWeather(epd, data, currentState):
    # WEATHER DISPLAY AND CONTROLLER FUNCTION
    current = data['current']

    # WEATHER CONTROLLER SECTION
    action = conditions(current['temp'], current['wind_speed'], current['wind_deg'])

    if action == "turnOff":
        if currentState == "on" or currentState == "unknown":
            currentState = turnOff()

    if action == "turnOn":
        if currentState == "off" or currentState == "unknown":
            currentState = turnOn()

    tz = pytz.timezone("America/Chicago")
    nowhour  = datetime.datetime.now(tz=tz).hour

    if action == "turnOnHalfTime":
        if nowhour > 21 or nowhour < 5:
            currentState = turnOn()
        else:
            if (nowhour % 2) == 0:
                if currentState == "off" or currentState == "unknown":
                    currentState = turnOn()
            else:
                if currentState == "on" or currentState == "unknown":
                    currentState = turnOff()

    # WEATHER DISPLAY SECTION
    daily = data['daily']
    weather = current['weather']
    hourly = data['hourly']

    now  = datetime.datetime.now(tz=tz) 
    updatedTime = now.strftime("%B %d, %Y %-I:%M %p")

    picdir = "/home/pi/pic"
    libdir = "/home/pi/lib"

    # set up fonts
    font12 = ImageFont.truetype(picdir + '/Font.ttc', 12)
    font14 = ImageFont.truetype(picdir + '/Font.ttc', 14)
    font16 = ImageFont.truetype(picdir + '/Font.ttc', 16)
    font18 = ImageFont.truetype(picdir + '/Font.ttc', 18)
    font24 = ImageFont.truetype(picdir + '/Font.ttc', 24)
    font35 = ImageFont.truetype(picdir + '/Font.ttc', 35)
    font48 = ImageFont.truetype(picdir + '/Font.ttc', 48)

    try:
        # set up screen image
        Limage = Image.new('1', (epd.height, epd.width), 255)
        draw = ImageDraw.Draw(Limage)

        # create 'last updated' section
        draw.text((20,1), "Last Updated", font = font14, fill=0)
        draw.text((115,1), updatedTime, font = font14, fill=0)

        # retrieve and draw weather icon
        URL = "http://openweathermap.org/img/wn/" +  weather[0]['icon'] + "@2x.png"

        try:
            response = requests.get(URL)

            if response.status_code == 200:
                with open(picdir + "/weathericon.png", 'wb') as f:
                    f.write(response.content)
                response.close()

                # convert transparent image to white background using imageMagick
                os.system('convert ' + picdir + '/weathericon.png -background white -flatten ' + picdir + '/weathericon.bmp')
                time.sleep(2)
                bmp = Image.open(picdir + "/weathericon.bmp")
                Limage.paste(bmp, (190,90))

            else:
                print("Error in icon request " + str(response.status_code))
        except ConnectionError as e:
            print("Icon retrieval error: ", e)

        # draw 'current conditions' section
        degree_symbol = u'\N{DEGREE SIGN}'

        draw.rectangle((0,23,299,50), outline=0)
        draw.text((40,23), "Current Conditions", font = font24, fill=0)
        draw.rectangle((0,50,299,73), outline=0)
        draw.text((12,50), "Temperature", font = font18, fill=0)
        draw.text((188,50), "Feels Like", font = font18, fill=0)
        draw.text((12,68), ('%.1f' % current['temp']) + degree_symbol, font = font48, fill=0)
        draw.text((180,68), ('%.1f' % current['feels_like']) + degree_symbol, font = font48, fill=0)

        bmp = Image.open(picdir +  '/wind.bmp')
        Limage.paste(bmp, (5,121))
        draw.text((47,118), str(round(current['wind_speed'])) + " MPH from the ", font = font16, fill=0)
        directionList = ["North", "NorthEast", "East", "SouthEast", "South", "SouthWest", "West", "NorthWest", "North"]
        direction = directionList[int((current['wind_deg'] + 22.5)/45)]
        draw.text((47,139), direction, font = font16, fill=0)

        bmp = Image.open(picdir +  '/sunrisesunset.bmp')
        Limage.paste(bmp, (1,156))
        sunrise = (current['sunrise'])
        sunset = (current['sunset'])
        draw.text((47,161), "Rise " + convert_time(sunrise) + " Set " + convert_time(sunset), font = font16, fill=0)
        description =  weather[0]['description'][0:20]
        draw.text((235 - (3 * len(description)),161), description, font = font14, fill=0)

        # draw 'hourly' section
        draw.rectangle((0,185,299,205), outline=0)
        draw.text((1,185), "Hour", font = font18, fill=0)
        draw.text((1,206), "Temp", font = font14, fill=0)
        draw.text((1,222), "Feels", font = font14, fill=0)

        for i in range(1,7):
            startX = 8
            startY = 187
            incrX = 40
            incrY = 0

            draw.text((startX + (incrX * i), startY + (incrY * i)), convert_hour(hourly[i]['dt']), font = font12, fill=0)

        for i in range(1,7):
            startX = 12
            startY = 206
            incrX = 40
            incrY = 16

            draw.text((startX + (incrX * i),startY), str(round(hourly[i]['temp'])), font = font14, fill=0)
            draw.text((startX + (incrX * i),startY + incrY), str(round(hourly[i]['feels_like'])), font = font14, fill=0)

        # draw 'tody and tonight' section
        draw.rectangle((0,244,299,288), outline=0)
        draw.text((6,245), "Today", font = font18, fill=0)
        draw.text((80,245), "High", font = font18, fill=0)
        draw.text((125,245), str(round(daily[0]['temp']['day'])), font = font18, fill=0)
        draw.text((168,245), "Feels Like", font = font18, fill=0)
        draw.text((256,245), str(round(daily[0]['feels_like']['day'])), font = font18, fill=0)
        draw.text((6,265), "Tonight", font = font18, fill=0)
        draw.text((80,265), "Low", font = font18, fill=0)
        draw.text((125,265), str(round(daily[0]['temp']['night'])), font = font18, fill=0)
        draw.text((168,265), "Feels Like", font = font18, fill=0)
        draw.text((256,265), str(round(daily[0]['feels_like']['night'])), font = font18, fill=0)

        # draw 'daily forecast' section
        draw.text((100,289), "Forecast", font = font18, fill=0)
        dailyweather = daily[0]['weather']
        draw.rectangle((0,311,299,329), outline=0)
        draw.text((4,311), "Day High Feels Low Feels  Description", font = font14, fill=0)

        for i in range(1, 4):
            startX = 45
            startY = 317
            incrX = 30
            incrY = 14

            dayofweek = datetime.datetime.fromtimestamp(int(daily[i]['dt'])).strftime('%A')[0:3]

            draw.text((2,startY + (incrY * i)), dayofweek, font = font14, fill=0)
            draw.text((startX,startY + (incrY * i)), str(round(daily[i]['temp']['day'])), font = font14, fill=0)
            draw.text((startX + incrX,startY + (incrY * i)), str(round(daily[i]['feels_like']['day'])), font = font14, fill=0)
            draw.text((startX + (incrX * 2),startY + (incrY * i)), str(round(daily[i]['temp']['night'])), font = font14, fill=0)
            draw.text((startX + (incrX * 3),startY + (incrY * i)), str(round(daily[i]['feels_like']['night'])), font = font14, fill=0)
            draw.text((startX + (incrX * 4),startY + (incrY * i)), daily[i]['weather'][0]['description'][0:20], font = font14, fill=0)

        # draw 'plug status' section
        draw.rectangle((0,378,299,399), outline=0)
        draw.text((3,378), "Fan:", font = font18, fill=0)
        draw.text((48,378), currentState, font = font18, fill=0)

        # display new image
        epd.init()
        epd.display(epd.getbuffer(Limage))
        epd.sleep()
    except IOError as e:
        print(e)

    except KeyboardInterrupt:
        epd4in2.epdconfig.module_exit()
        exit()

    return(currentState)


def conditions(temp, speed, degree):
    # WEATHER CONTROLLER FUNCTION
    if temp > 45:
        return "turnOff" 

    if speed > 15:
        return "turnOff"

    if speed > 4:
        if degree < 90  or degree > 270:
            return "turnOff"
        else:
            return "turnOnHalfTime"

    return "turnOn"

def turnOn():
    # WEATHER CONTROLLER FUNCTION 
    retrieved = False

    for i in range(0,5):
        try:
            response = requests.post("https://maker.ifttt.com/trigger/TurnOnFan/with/key/hIS_w6dXli-279redZ1h8jRi01LZoJVH0BjP4xmnuZB")

            if response.status_code == 200:
                print("Response text: ", response.text)
                if "Congratulations!" in response.text:
                     print("TURNED FAN ON!")
                     retrieved = True
                else:
                    print("Wrong text returned in Turn On request: ", response.text)
                response.close()
            else:
                response.close()
                print("Error in the Turn On request " + str(response.status_code))

        except ConnectionError as e:
            print("Turn on error: ", e)

        if retrieved == True:
            return("on")

        time.sleep(20)

    sleep(300)
    return("off")

def turnOff():
    # WEATHER CONTROLLER FUNCTION
 
    retrieved = False

    for i in range(0,5):
        try:
            response = requests.post("https://maker.ifttt.com/trigger/TurnOffFan/with/key/hIS_w6dXli-279redZ1h8jRi01LZoJVH0BjP4xmnuZB")

            if response.status_code == 200:
                print("Response text: ", response.text)
                if "Congratulations!" in response.text:
                     print("TURNED FAN OFF!")
                     retrieved = True
                else:
                    print("Wrong text returned in Turn Off request: ", response.text)
                response.close()
                print("TURNED FAN OFF!")
                retrieved = True
            else:
                response.close()
                print("Error in the Turn Off request " + str(response.status_code))

        except ConnectionError as e:
            print("Turn off error: ", e)

        if retrieved == True:
            return("off")

        time.sleep(20)

    return("on")

def main():
    # MAIN PROGRAM
    BASE_URL = "https://api.openweathermap.org/data/2.5/onecall?"
    PARM_URL = "lat=41.97928018230549&lon=-91.73275790771059&appid=a4b2f480089f3657764f97dab223992d&units=imperial"

    currentPlugState = "unknown"

    epd = setup_screen()

    while True:
        print("Processing...")
        retrieved = False

        # Get weather

        for i in range(0,10):
            try:
                response = requests.get(BASE_URL+PARM_URL)
                if response.status_code == 200:
                    retrieved = True
                    data = response.json()
                    response.close()
                    currentPlugState = useCurrentWeather(epd, data, currentPlugState)
                else:
                    print("Error in the One Call request " + str(response.status_code))
            except ConnectionError as e:
                print("One Call error: ", e)

            if retrieved == True:
                break

            time.sleep(10)

        time.sleep(300)

if __name__ == '__main__':
    main()
